==> afflib_3.0.0a4.orig.tar.gz: afflib-3.0.0a4/tools/afsign.cpp <==
/*
 * afsign.cpp:
 *
 * Sign an existing AFF file.
 */

/*
 * Copyright (c) 2007
 *	Simson L. Garfinkel and Basis Technology, Inc. 
 *      All rights reserved.
 *
 * This code is derrived from software contributed by
 * Simson L. Garfinkel
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *    This product includes software developed by Simson L. Garfinkel
 *    and Basis Technology Corp.
 * 4. Neither the name of Simson Garfinkel, Basis Technology, or other
 *    contributors to this program may be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY SIMSON GARFINKEL, BASIS TECHNOLOGY,
 * AND CONTRIBUTORS ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SIMSON GARFINKEL, BAIS TECHNOLOGy,
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.  
 */

#include "config.h"
#include "afflib.h"
#include "afflib_i.h"

#include "utils.h"
#include "base64.h"

#include <stdio.h>
#include <algorithm>
#include <vector>
#include <iostream>
#include <openssl/pem.h>
#include <openssl/x509.h>

#ifdef USE_AFFSIGS

int opt_note = 0;
const char *opt_sign_key_file = 0;
const char *opt_sign_cert_file = 0;

using namespace std;

char *progname = "afsign";

void usage()
{
    printf("%s version %s\n",progname,PACKAGE_VERSION);
    printf("usage: %s [options] filename.aff\n",progname);
    printf("This program will:\n");
    printf("  * Sign each segment if there are no segment signatures.\n");
    printf("  * Write signed chain-of-custody Bill of Materials segment.\n");
    printf("\nSignature Options:\n");
    printf("   -s filename.key   = specify private key for signing\n");
    printf("   -c filename.cer   = specify a X.509 certificate that matches the private key\n");
    printf("                       (by default, the file is assumed to be the same one\n");
    printf("                       provided with the -s option.)");
    printf("options:\n");
    printf("    -n      --- ask for a chain-of-custody note.\n");
    printf("    -v      --- Just print the version number and exit.\n");
    exit(0);
}

void make_hash(u_char seghash[32],
	     unsigned long arg,const char *segname,
	     const u_char *pagebuf,
	     unsigned long pagesize)
{
    unsigned int seghash_len = sizeof(seghash);
    unsigned long arg_net = htonl(arg);
    EVP_MD_CTX md;		/* EVP message digest */
    EVP_DigestInit(&md,EVP_sha256());
    EVP_DigestUpdate(&md,(const unsigned char *)segname,strlen(segname)+1);
    EVP_DigestUpdate(&md,(const unsigned char *)&arg_net,sizeof(arg_net));
    EVP_DigestUpdate(&md,pagebuf,pagesize);
    EVP_DigestFinal(&md,seghash,&seghash_len);
}

int afsign(const char *fn)
{
    AFFILE *af = af_open(fn,O_RDWR,0);
    if(!af) err(1,"%s",fn);

    struct af_vnode_info vni;
    if(af_vstat(af,&vni)) err(1,"af_vstat");

    if(vni.supports_metadata==0){
	/* If it is a raw file, we can create an AFM file to sign */
	if(vni.is_raw==0) errx(1,"%s: file does not support metadata. Cannot sign\n",fn);
	af_close(af);			// afm will open it
	char afmfile[MAXPATHLEN+1];
	char file000[MAXPATHLEN+1];
	char extension[MAXPATHLEN+1];
	strcpy(afmfile,fn);
	char *period = strrchr(afmfile,'.');
	if(!period) errx(1,"%s: file does not support metadata and lacks a file extension,\n"
			 "which is needed to create an AFM file '%s\n",fn);
	strcpy(extension,period+1);	// get the extension

	/* If the file being opened is not a .000 file, and a .000 file exists, do not proceed */
	strcpy(period,".000");
	strcpy(file000,afmfile);	// make the 000 file
	strcpy(period,".afm");
	
	if(strcmp(extension,"000")!=0){
	    if(access(file000,F_OK)==0){
		errx(1,"Can't create .afm file because %s exists.\n",file000);
	    }
	}
	
	strcpy(period,".afm");		// we are now going to make an afm file
	af = af_open(afmfile,O_RDWR|O_CREAT,0600);
	if(!af) errx(1,"%s: file does not support metadata and cannot create AFM file '%s\n",fn,afmfile);
	if(strcmp(extension,"000")!=0){
	    af_update_seg(af,AF_RAW_IMAGE_FILE_EXTENSION,0,(const u_char *)extension,strlen(extension));
	    af_close(af);
	    unlink(file000);		// get rid of that .000 file
	    af = af_open(afmfile,O_RDWR,0600);
	    if(!af) errx(1,"%s: Created AFM file but cannot re-open it\n",fn);
	    /* Read the first byte to force a call to afm_split_raw_setup().
	     * The results of the read don't matter, but we better be able to read.
	     */
	    u_char buf[1];
	    if(af_read(af,buf,1)!=1){
		err(1,"Cannot read first byte of %s",fn);
	    }
	    af_seek(af,0L,0);
	}
    }

    seglist segments(af);

    if(isatty(fileno(stdout))){
	printf("Signing segments...\n");
	fflush(stdout);
    }
    
    if(segments.has_signed_segments()==false){
	if(af_set_sign_files(af,opt_sign_key_file,opt_sign_cert_file)){
	    err(1,opt_sign_key_file);
	}
	if(af_sign_all_unsigned_segments(af)){
	    err(1,"%s: all unsigned segments cannot be signed.",fn);
	}
    }

    aff_bom bom(opt_note);
    if(bom.read_files(opt_sign_cert_file,opt_sign_key_file)) err(1,"Can't read signature files???");

    size_t pagesize = af_page_size(af);
    u_char *pagebuf = (unsigned char *)malloc(pagesize);
    u_char *parity_buf = 0;

    /* Create the parity buffer if it doesn't exist.
     */
    if(af_get_seg(af,AF_PARITY0,0,0,0)!=0){
	parity_buf = (unsigned char *)calloc(pagesize,1);
    }

    for(seglist::const_iterator seg = segments.begin(); seg!= segments.end();seg++){
	const char *segname = seg->name.c_str();

	if(isatty(fileno(stdout))){
	    printf("\rCalculating BOM for page %s...                          ",segname);
	    fflush(stdout);
	}

	u_char seghash[32]; /* resultant message digest; could be any size */
	unsigned int seghash_len = sizeof(seghash); /* big enough to hold SHA256 */
	int sigmode = 0;
	int64 pagenumber = af_segname_page_number(segname);
	if(pagenumber>=0){
	    pagesize = af_page_size(af);
	    if(af_get_page(af,pagenumber,pagebuf,&pagesize)){ 
		free(pagebuf);
		return -1;
	    }
	    /* Add to parity buf */
	    for(int i=0;parity_buf && i<pagesize;i++){
		parity_buf[i] ^= pagebuf[i];
	    }
	    make_hash(seghash,0,segname,pagebuf,pagesize);
	    sigmode = AF_SIGNATURE_MODE1;
	}
	else{
	    size_t seglen=0;
	    if(af_get_seg(af,segname,0,0,&seglen)){
		err(1,"Cannot read length of segment '%s' on input file %s", segname,af_filename(af));
	    }
	    unsigned char *segbuf = (unsigned char *)malloc(seglen);
	    if(!segbuf){
		err(1,"Cannot allocated %d bytes for segment '%s' in %s",
		    (int)seglen,segname,af_filename(af));
	    }
	    /* Now get the raw source segment */
	    unsigned long arg=0;
	    if(af_get_seg(af,segname,&arg,segbuf,&seglen)){
		err(1,"Cannot read segment '%s' in %s. Deleteing output file", segname,af_filename(af));
	    }
	    make_hash(seghash,arg,segname,pagebuf,pagesize);
	    sigmode = AF_SIGNATURE_MODE0;
	    free(segbuf);
	}
	bom.add(segname,sigmode,seghash,seghash_len);
    }

    /* If we have been making the parity buf we need write it out,
     * and then add both it and its signature to the bom
     */
    if(parity_buf){
	u_char buf[1024];
	size_t buflen = sizeof(buf);
	u_char seghash[32]; /* resultant message digest; could be any size */
	unsigned int seghash_len = sizeof(seghash); /* big enough to hold SHA256 */
	
	if(af_update_seg(af,AF_PARITY0,0,parity_buf,pagesize)) err(1,"Can't write %s",AF_PARITY0);
	make_hash(seghash,0,AF_PARITY0,parity_buf,pagesize);
	bom.add(AF_PARITY0,AF_SIGNATURE_MODE0,seghash,seghash_len);
	const char *segname = AF_PARITY0 AF_SIG256_SUFFIX;
	if(af_get_seg(af,segname,0,buf,&buflen)) err(1,"Can't read %s",segname);
	make_hash(seghash,0,segname,buf,buflen);
	bom.add(segname,AF_SIGNATURE_MODE0,seghash,seghash_len);
    }
    
    if(isatty(fileno(stdout))){
	printf("                                  \r");
	fflush(stdout);
    }
    

    bom.close();
    bom.write(af,segments);
    af_close(af);
    return 0;
}

int main(int argc,char **argv)
{
    int bflag, ch;

    bflag = 0;
    while ((ch = getopt(argc, argv, "ns:c:h?v")) != -1) {
	switch (ch) {
	case 'n': opt_note = 1;break;
	case 's':
	    if(access(optarg,R_OK)) err(1,optarg);
	    opt_sign_key_file = optarg;
	    if(opt_sign_cert_file==0) opt_sign_cert_file=optarg;
	    break;
	case 'c':
	    if(access(optarg,R_OK)) err(1,optarg);
	    opt_sign_cert_file = optarg;
	    break;
	case 'v':
	    printf("%s version %s\n",progname,PACKAGE_VERSION);
	    exit(0);
	case 'h':
	case '?':
	default:
	    usage();
	    break;
	}
    }
    argc -= optind;
    argv += optind;

    if(argc!=1){
	usage();
    }

    /* We either need both a key file and a cert file, or neither */
    if((opt_sign_key_file==0) || (opt_sign_cert_file==0)){
	errx(1,"Both a private key and a certificate must be specified.");
    }

    return afsign(argv[0]);
}
#else
int main(int argc,char **argv)
{
    errx(1,"%s cannot run because AFFLIB was compiled without AFF signature support",argv[0]);
}
#endif

==> afflib-3.2.0.tar.gz: afflib-3.2.0/tools/afsign.cpp <==
/*
 * afsign.cpp:
 *
 * Sign an existing AFF file.
 */

/*
 * Copyright (c) 2007
 *	Simson L. Garfinkel and Basis Technology, Inc. 
 *      All rights reserved.
 *
 * This code is derrived from software contributed by
 * Simson L. Garfinkel
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *    This product includes software developed by Simson L. Garfinkel
 *    and Basis Technology Corp.
 * 4. Neither the name of Simson Garfinkel, Basis Technology, or other
 *    contributors to this program may be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * THIS SOFTWARE IS PROVIDED BY SIMSON GARFINKEL, BASIS TECHNOLOGY,
 * AND CONTRIBUTORS ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL SIMSON GARFINKEL, BAIS TECHNOLOGy,
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.  
 */

#include "affconfig.h"
#include "afflib.h"
#include "afflib_i.h"
#include "afflib_sha256.h"

#include "utils.h"
#include "base64.h"

#include <stdio.h>
#include <algorithm>
#include <vector>
#include <iostream>
#include <openssl/pem.h>
#include <openssl/x509.h>

#include "aff_bom.h"

#ifdef USE_AFFSIGS

int opt_note = 0;
const char *opt_sign_key_file = 0;
const char *opt_sign_cert_file = 0;

using namespace std;
using namespace aff;

const char *progname = "afsign";

void usage()
{
    printf("%s version %s\n",progname,PACKAGE_VERSION);
    printf("usage: %s [options] filename.aff\n",progname);
    printf("This program will:\n");
    printf("  * Sign each segment if there are no segment signatures.\n");
    printf("  * Write signed chain-of-custody Bill of Materials segment.\n");
    printf("\nSignature Options:\n");
    printf("   -k filename.key   = specify private key for signing\n");
    printf("   -c filename.cer   = specify a X.509 certificate that matches the private key\n");
    printf("                       (by default, the file is assumed to be the same one\n");
    printf("                       provided with the -k option.)");
    printf("options:\n");
    printf("    -n      --- ask for a chain-of-custody note.\n");
    printf("    -v      --- Just print the version number and exit.\n");
    exit(0);
}

void make_hash(u_char seghash[32],
	     unsigned long arg,const char *segname,
	     const u_char *pagebuf,
	     unsigned long pagesize)
{
    unsigned int seghash_len = sizeof(seghash);
    unsigned long arg_net = htonl(arg);
    EVP_MD_CTX md;		/* EVP message digest */
    EVP_DigestInit(&md,EVP_sha256());
    EVP_DigestUpdate(&md,(const unsigned char *)segname,strlen(segname)+1);
    EVP_DigestUpdate(&md,(const unsigned char *)&arg_net,sizeof(arg_net));
    EVP_DigestUpdate(&md,pagebuf,pagesize);
    EVP_DigestFinal(&md,seghash,&seghash_len);
}

int afsign(const char *fn)
{
    AFFILE *af = af_open(fn,O_RDWR,0);
    if(!af) err(1,"%s",fn);

    struct af_vnode_info vni;
    if(af_vstat(af,&vni)) err(1,"af_vstat");

    if(vni.supports_metadata==0){
	/* If it is a raw file, we can create an AFM file to sign */
	if(vni.is_raw==0) errx(1,"%s: file does not support metadata. Cannot sign\n",fn);
	af_close(af);			// afm will open it
	char afmfile[MAXPATHLEN+1];
	char file000[MAXPATHLEN+1];
	char extension[MAXPATHLEN+1];
	strcpy(afmfile,fn);
	char *period = strrchr(afmfile,'.');
	if(!period) errx(1,"%s: file does not support metadata and lacks a file extension,\n"
			 "which is needed to create an AFM file '%s\n",fn);
	strcpy(extension,period+1);	// get the extension

	/* If the file being opened is not a .000 file, and a .000 file exists, do not proceed */
	strcpy(period,".000");
	strcpy(file000,afmfile);	// make the 000 file
	strcpy(period,".afm");
	
	if(strcmp(extension,"000")!=0){
	    if(access(file000,F_OK)==0){
		errx(1,"Can't create .afm file because %s exists.\n",file000);
	    }
	}
	
	strcpy(period,".afm");		// we are now going to make an afm file
	af = af_open(afmfile,O_RDWR|O_CREAT,0600);
	if(!af) errx(1,"%s: file does not support metadata and cannot create AFM file '%s\n",fn,afmfile);
	if(strcmp(extension,"000")!=0){
	    af_update_seg(af,AF_RAW_IMAGE_FILE_EXTENSION,0,(const u_char *)extension,strlen(extension));
	    af_close(af);
	    unlink(file000);		// get rid of that .000 file
	    af = af_open(afmfile,O_RDWR,0600);
	    if(!af) errx(1,"%s: Created AFM file but cannot re-open it\n",fn);
	    /* Read the first byte to force a call to afm_split_raw_setup().
	     * The results of the read don't matter, but we better be able to read.
	     */
	    u_char buf[1];
	    if(af_read(af,buf,1)!=1){
		err(1,"Cannot read first byte of %s",fn);
	    }
	    af_seek(af,0L,0);
	}
    }

    seglist segments(af);

    if(isatty(fileno(stdout))){
	printf("Signing segments...\n");
	fflush(stdout);
    }
    
    if(segments.has_signed_segments()==false){
	if(af_set_sign_files(af,opt_sign_key_file,opt_sign_cert_file)){
	    errx(1,"afsign: key file '%s' or certificate file '%s' is invalid",
		 opt_sign_key_file,opt_sign_cert_file);
	}
	if(af_sign_all_unsigned_segments(af)){
	    err(1,"%s: all unsigned segments cannot be signed.",fn);
	}
    }

    aff_bom bom(opt_note);
    if(bom.read_files(opt_sign_cert_file,opt_sign_key_file)) err(1,"Can't read signature files???");

    u_char *pagebuf = (unsigned char *)calloc(af_page_size(af),1);
    u_char *parity_buf = 0;

    /* Create the parity buffer if it doesn't exist.
     */
    if(af_get_seg(af,AF_PARITY0,0,0,0)!=0){
	parity_buf = (unsigned char *)calloc(af_page_size(af),1);
    }

    for(seglist::const_iterator seg = segments.begin(); seg!= segments.end();seg++){
	const char *segname = seg->name.c_str();

	if(isatty(fileno(stdout))){
	    printf("\rCalculating BOM for page %s...                          ",segname);
	    fflush(stdout);
	}

	u_char seghash[32]; /* resultant message digest; could be any size */
	unsigned int seghash_len = sizeof(seghash); /* big enough to hold SHA256 */
	int sigmode = 0;
	int64_t pagenumber = af_segname_page_number(segname);
	if(pagenumber>=0){
	    size_t this_pagesize = af_page_size(af);
	    if(af_get_page(af,pagenumber,pagebuf,&this_pagesize)){ 
		free(pagebuf);
		return -1;
	    }
	    /* Add to parity buf */
	    for(int i=0;parity_buf && i<this_pagesize;i++){
		parity_buf[i] ^= pagebuf[i];
	    }
	    make_hash(seghash,0,segname,pagebuf,this_pagesize);
	    sigmode = AF_SIGNATURE_MODE1;
	}
	else{
	    size_t seglen=0;
	    if(af_get_seg(af,segname,0,0,&seglen)){
		err(1,"Cannot read length of segment '%s' on input file %s", segname,af_filename(af));
	    }
	    unsigned char *segbuf = (unsigned char *)malloc(seglen);
	    if(!segbuf){
		err(1,"Cannot allocated %d bytes for segment '%s' in %s",
		    (int)seglen,segname,af_filename(af));
	    }
	    /* Now get the raw source segment */
	    unsigned long arg=0;
	    if(af_get_seg(af,segname,&arg,segbuf,&seglen)){
		err(1,"Cannot read segment '%s' in %s. Deleteing output file", segname,af_filename(af));
	    }
	    make_hash(seghash,arg,segname,segbuf,seglen);
	    sigmode = AF_SIGNATURE_MODE0;
	    free(segbuf);
	}
	bom.add(segname,sigmode,seghash,seghash_len);
    }

    /* If we have been making the parity buf we need write it out,
     * and then add both it and its signature to the bom
     */
    if(parity_buf){
	u_char buf[1024];
	size_t buflen = sizeof(buf);
	u_char seghash[32]; /* resultant message digest; could be any size */
	unsigned int seghash_len = sizeof(seghash); /* big enough to hold SHA256 */
	
	if(af_update_seg(af,AF_PARITY0,0,parity_buf,af_page_size(af))) err(1,"Can't write %s",AF_PARITY0);
	make_hash(seghash,0,AF_PARITY0,parity_buf,af_page_size(af));
	bom.add(AF_PARITY0,AF_SIGNATURE_MODE0,seghash,seghash_len);

	const char *segname = AF_PARITY0 AF_SIG256_SUFFIX;
	if(af_get_seg(af,segname,0,buf,&buflen)) errx(1,"Can't read %s",segname);
	make_hash(seghash,0,segname,buf,buflen);
	bom.add(segname,AF_SIGNATURE_MODE0,seghash,seghash_len);
    }
    
    if(isatty(fileno(stdout))){
	printf("                                                    \r\n");
	fflush(stdout);
    }
    

    bom.close();
    bom.write(af,segments);
    af_close(af);
    return 0;
}

int main(int argc,char **argv)
{
    int bflag, ch;

    bflag = 0;
    while ((ch = getopt(argc, argv, "nk:c:h?v")) != -1) {
	switch (ch) {
	case 'n': opt_note = 1;break;
	case 'k':
	    if(access(optarg,R_OK)) err(1,optarg);
	    opt_sign_key_file = optarg;
	    break;
	case 'c':
	    if(access(optarg,R_OK)) err(1,optarg);
	    opt_sign_cert_file = optarg;
	    break;
	case 'v':
	    printf("%s version %s\n",progname,PACKAGE_VERSION);
	    exit(0);
	case 'h':
	case '?':
	default:
	    usage();
	    break;
	}
    }
    argc -= optind;
    argv += optind;

    if(opt_sign_cert_file==0) opt_sign_cert_file=opt_sign_key_file; // if not set, make same as key file


    if(argc!=1){
	usage();
    }

    /* We either need both a key file and a cert file, or neither */
    if((opt_sign_key_file==0) || (opt_sign_cert_file==0)){
	errx(1,"Both a private key and a certificate must be specified.");
    }

    return afsign(argv[0]);
}
#else
int main(int argc,char **argv)
{
    fprintf(stderr,"%s cannot run because AFFLIB was compiled without AFF signature support.\n",argv[0]);
    fprintf(stderr,"Signature support requires OPENSSL EVP SHA256 and lib expat\n");
    fprintf(stderr,"For further information, please http://www.afflib.org/requirements.php\n");
#ifndef HAVE_EVP_SHA256
    fprintf(stderr,"*** EVP_SHA256 was not found when configure was run\n");
#endif
#ifndef HAVE_LIBEXPAT
    fprintf(stderr,"*** LIBEXPAT was not found when configure was run\n");
#endif    
    exit(1);
}
#endif
